#!/usr/bin/env python

__RCSID__ = "$Id$"

import DIRAC
from DIRAC.Core.Base.Script import Script

Script.setUsageMessage(
    "\n".join(
        [
            "Get storage elements usage of production SEs",
            "Usage:",
            "%s <file with output of lcg-infosites --vo vo.cta.in2p3.fr se>"
            % Script.scriptName,
            f"\ne.g: {Script.scriptName} ccsrm02.in2p3.fr (default is all SEs)",
        ]
    )
)


Script.parseCommandLine(ignoreErrors=True)

from DIRAC.Core.Utilities.PrettyPrint import printTable
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations


@Script()
def main():
    args = Script.getPositionalArgs()

    if len(args) > 2:
        Script.showHelp()

    # Default list
    if len(args) == 1:
        opsHelper = Operations()
        SEList = opsHelper.getValue("ProductionSEs/Hosts", [])
    else:
        SEList = args[:2]

    sedict = {}
    for SE in SEList:
        sedict[SE] = []

    fields = ["SE", "Available(TB)", "Used(TB)", "Total(TB)", "Used(%)"]
    records = []

    fp = open(args[0])

    for se in fp:
        if len(se.split()) == 4:
            spacedict = {}
            SE = se.split()[3]
            if SE in SEList and se.split()[0] != "n.a" and se.split()[1] != "n.a":
                # ## convert into TB
                available = float(se.split()[0]) / 1e9
                used = float(se.split()[1]) / 1e9
                spacedict["Available"] = available
                spacedict["Used"] = used
                spacedict["Total"] = available + used
                sedict[SE].append(spacedict)

    for SE in SEList:
        for spacedict in sedict[SE]:
            available = f"{spacedict['Available']:.1f}"
            used = f"{spacedict['Used']:.1f}"
            total = f"{spacedict['Total']:.1f}"
            fraction_used = spacedict["Used"] / spacedict["Total"] * 100
            fraction_used = f"{fraction_used:.1f}"
            records.append([SE, available, used, total, fraction_used])

    printTable(fields, records)
    DIRAC.exit()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
Get statistics for one or more datasets

Warning: --SEUsage option gives the storage report per SE
         and it assumes that datasets are defined with directory level metadata only
Usage:
   cta-prod-show-dataset <datasetName (may contain wild cards) or ascii file with a list of datasets> <options>
"""

__RCSID__ = "$Id$"

import os

import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Base.Script import Script
from DIRAC.Core.Utilities.PrettyPrint import int_with_commas, printTable
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from CTADIRAC.Core.Utilities.tool_box import read_inputs_from_file


def print_dataset_list(dataset_list):
    dataset_list.sort()
    print(f"\nAvailable datasets: {len(dataset_list)}\n")
    for dataset_name in dataset_list:
        gLogger.notice(dataset_name)
    gLogger.notice("\n")


def get_list_of_datasets(tag=""):
    fc = FileCatalogClient()
    dataset_tag = f"*{tag}*"
    res = fc.getDatasets(dataset_tag)
    if not res["OK"]:
        gLogger.error(res["Message"])
        DIRAC.exit(-1)
    dataset_dict = res["Value"]
    dataset_list = list()
    for dataset_name in dataset_dict["Successful"][dataset_tag]:
        dataset_list.append(dataset_name)
    return dataset_list


def get_dataset_info(dataset_name):
    fc = FileCatalogClient()
    res = fc.getDatasets(dataset_name)
    if not res["OK"]:
        gLogger.error("Failed to get datasets")
        DIRAC.exit(-1)
    dataset_dict = res["Value"]
    res = dataset_dict["Successful"][dataset_name][dataset_name]
    number_of_files = res["NumberOfFiles"]
    meta_query = res["MetaQuery"]
    total_size = res["TotalSize"]
    return (dataset_name, number_of_files, total_size, meta_query)


def print_dataset_storage_usage(dataset_name):
    fc = FileCatalogClient()
    fields = ["StorageElement", "Size", "Replicas"]
    totalSize = 0.0
    totalFiles = 0
    name, n_files, size, mq = get_dataset_info(dataset_name)
    outputType = mq["outputType"]
    result = fc.findDirectoriesByMetadata(mq)
    if not result["OK"]:
        gLogger.error(result["Message"])
        DIRAC.exit(-1)
    gLogger.notice("Directory size")
    for k, path in result["Value"].items():
        if os.path.basename(path) == outputType:
            gLogger.notice(path)
            result = fc.getDirectorySize(path, longOut=True)
            values = []
            res_path_dict = result["Value"]["Successful"][path]
            if "PhysicalSize" in res_path_dict:
                if "TotalSize" in res_path_dict["PhysicalSize"]:
                    totalSize = res_path_dict["PhysicalSize"]["TotalSize"]
                if "TotalFiles" in res_path_dict["PhysicalSize"]:
                    totalFiles = res_path_dict["PhysicalSize"]["TotalFiles"]
                for se, sdata in result["Value"]["Successful"][path][
                    "PhysicalSize"
                ].items():
                    if not se.startswith("Total"):
                        size = sdata["Size"]
                        nfiles = sdata["Files"]
                        values.append((se, int_with_commas(size), str(nfiles)))
                values.append(("Total", int_with_commas(totalSize), str(totalFiles)))
                printTable(fields, values)


def print_dataset_storage_usage_summary(path_list):
    fc = FileCatalogClient()
    fields = ["StorageElement", "Size", "Replicas"]
    values = []
    se_list = []
    stat_dict = {}
    total_stat_dict = {"Total": {"size": 0, "nFiles": 0}}

    # Initialize the list of SE for all paths
    for path in path_list:
        result = fc.getDirectorySize(path, longOut=True)
        if "PhysicalSize" in result["Value"]["Successful"][path]:
            for se, sdata in result["Value"]["Successful"][path][
                "PhysicalSize"
            ].items():
                if not se.startswith("Total"):
                    se_list.append(se)

    # Initialize the total_stat_dict with all SE keys
    se_list = sorted(set(se_list))
    for se in se_list:
        total_stat_dict.update({se: {"size": 0, "nFiles": 0}})
        stat_dict.update({se: {"size": 0.0, "nFiles": 0}})

    # For each path get the directory usage and calculate the total usage for each SE
    # and the overall total usage
    for path in path_list:
        for se in se_list:
            stat_dict[se]["size"] = 0
            stat_dict[se]["nFiles"] = 0
        result = fc.getDirectorySize(path, longOut=True)
        res_path_dict = result["Value"]["Successful"][path]
        if "PhysicalSize" in res_path_dict:
            for se, sdata in result["Value"]["Successful"][path][
                "PhysicalSize"
            ].items():
                if not se.startswith("Total"):
                    stat_dict[se]["size"] = sdata["Size"]
                    stat_dict[se]["nFiles"] = sdata["Files"]

        # Calculate the total usage for each SE
        for se in stat_dict:
            total_stat_dict[se]["size"] += stat_dict[se]["size"]
            total_stat_dict[se]["nFiles"] += stat_dict[se]["nFiles"]

    # Calculate the overall total usage
    for se in stat_dict:
        total_stat_dict["Total"]["size"] += total_stat_dict[se]["size"]
        total_stat_dict["Total"]["nFiles"] += total_stat_dict[se]["nFiles"]

    # Update the values of the table for each SE
    for se in se_list:
        values.append(
            (
                se,
                int_with_commas(total_stat_dict[se]["size"]),
                str(total_stat_dict[se]["nFiles"]),
            )
        )

    # Update the values of the table with the Total
    values.append(
        (
            "Total",
            int_with_commas(total_stat_dict["Total"]["size"]),
            str(total_stat_dict["Total"]["nFiles"]),
        )
    )

    printTable(fields, values)


#########################################################
@Script()
def main():
    Script.registerSwitch("l", "long", "print dataset details")
    Script.registerSwitch("", "SEUsage", "print storage usage summary per SE")
    switches, argss = Script.parseCommandLine(ignoreErrors=True)

    if len(argss) == 0:
        dataset_list = get_list_of_datasets()
        print_dataset_list(dataset_list)
        DIRAC.exit()
    elif len(argss) == 1:
        if os.path.isfile(argss[0]):
            gLogger.notice("Reading datasets from input file:", argss[0])
            dataset_list = read_inputs_from_file(argss[0])
        else:
            dataset_name = argss[0]
            dataset_list = list()
            if dataset_name.find("*") > 0:
                dataset_list = get_list_of_datasets(dataset_name)
            else:
                dataset_list.append(dataset_name)
        print_dataset_list(dataset_list)

    long = False
    SEUsage = False
    for switch in switches:
        if "l" == switch[0] or "long" == switch[0]:
            long = True
        if "SEUsage" == switch[0]:
            SEUsage = True

    if SEUsage:
        gLogger.notice("Getting statistics informations for all datasets\n")

    # Get dataset info only if SE usage report or dataset details are requested
    if SEUsage or long:
        path_list = []
        fc = FileCatalogClient()

        ## Check FileMetadata
        res = fc.getMetadataFields()
        if not res["OK"]:
            gLogger.error(res["Message"])
            DIRAC.exit(-1)
        fileMetaFields = res["Value"]["FileMetaFields"]

        i = 1
        for dataset_name in dataset_list:
            name, n_files, size, mq = get_dataset_info(dataset_name)
            if long:
                gLogger.notice("%d) Dataset name: %s\n" % (i, name))
                gLogger.notice("MetaQuery")
                gLogger.notice(str(mq), "\n")
                i += 1
            # Get the path list only if the SE usage report is requested
            if SEUsage:
                for metadata in mq:
                    if metadata in fileMetaFields:
                        gLogger.error("Warning: File level metadata found:", metadata)
                        gLogger.error(
                            "Storage report per SE could be wrong for dataset:",
                            dataset_name,
                        )
                if long:
                    print_dataset_storage_usage(dataset_name)
                if "outputType" in mq:
                    outputType = mq["outputType"]
                else:
                    gLogger.error(
                        "Error getting SEUsage, no outputType metadata defined"
                    )
                    gLogger.error("for dataset:", name)
                    DIRAC.exit(-1)
                result = fc.findDirectoriesByMetadata(mq)
                for k, path in result["Value"].items():
                    if os.path.basename(path) == outputType:
                        path_list.append(path)

    if SEUsage:
        gLogger.notice("\nStorage usage per SE for all datasets\n")
        print_dataset_storage_usage_summary(path_list)

    # To do: optimize to avoid calling twice get_dataset_info when using SEUsage or long options
    gLogger.notice("Storage usage for all datasets\n")
    gLogger.notice("|_. Name |_. N files |_. Size(TB) |")
    total_size = 0.0
    total_n_files = 0
    for dataset_name in dataset_list:
        name, n_files, size, mq = get_dataset_info(dataset_name)
        # # convert total size in TB
        size_TB = size / 1e12
        gLogger.notice("|%s|%d|%.2f|" % (name, n_files, size_TB))
        total_size += size_TB
        total_n_files += n_files
    gLogger.notice("| Total | %d | %.2f |\n" % (total_n_files, total_size))

    DIRAC.exit()


if __name__ == "__main__":
    main()



"""
    Test for the Client
"""
__RCSID__ = "$Id$"

import DIRAC
from DIRAC.Core.Base import Script


Script.setUsageMessage("\n".join([__doc__.split("\n")[1]]))

Script.parseCommandLine()

from ProvClient import ProvClient
from ProvBase import Agent
from ProvBase import ActivityDescription
from ProvBase import DatasetDescription
from ProvBase import UsageDescription
from ProvBase import GenerationDescription
from ProvBase import ValueDescription
from ProvBase import DatasetEntity
from ProvBase import WasAttributedTo
from ProvBase import Used
from ProvBase import WasGeneratedBy
from ProvBase import ValueEntity
from ProvBase import Activity
from ProvBase import WasAssociatedWith

provClient = ProvClient()

# Create an instance of ActivityDescription
actDesc1 = ActivityDescription(
    id="ctapipe_display_muons_0.6.1",
    name="ctapipe_display_muons",
    description="",
    type="",
    subtype="",
    version="0.6.1",
    doculink="",
)
# Create the description of input entities
dataDesc1 = DatasetDescription(
    id="proton_events",
    name="protons",
    description="proton file",
    classType="datasetDescription",
)
usedDesc1 = UsageDescription(
    id="ctapipe_display_muons_0.6.1_proton_events",
    activityDescription_id=actDesc1.id,
    entityDescription_id=dataDesc1.id,
    role="dl0.sub.evt",
)
# Create the description of output entities
dataDesc2 = DatasetDescription(
    id="muons_hdf5",
    name="muons",
    description="muon file",
    classType="datasetDescription",
)
wGBDesc1 = GenerationDescription(
    id="ctapipe_display_muons_0.6.1_muons_hdf5",
    activityDescription_id=actDesc1.id,
    entityDescription_id=dataDesc2.id,
    role="dl0.sub.evt",
)
valueDesc1 = ValueDescription(id="status", classType="valueDescription")
wGBDesc2 = GenerationDescription(
    id="ctapipe_display_muons_0.6.1_status",
    activityDescription_id=actDesc1.id,
    entityDescription_id=valueDesc1.id,
    role="quality",
)

#
res = provClient.addActivityDescription(actDesc1)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)
res = provClient.addDatasetDescription(dataDesc1)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)
res = provClient.addUsageDescription(usedDesc1)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)
res = provClient.addDatasetDescription(dataDesc2)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)
res = provClient.addGenerationDescription(wGBDesc1)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)
res = provClient.addValueDescription(valueDesc1)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)
res = provClient.addGenerationDescription(wGBDesc2)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)


# CTAO Agent
agent = Agent(id="CTAO")
agent.name = "CTA Observatory"
agent.type = "Organization"

res = provClient.addAgent(agent)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)


current_activity = Activity(id="ed7e27d4-a0a7-43a2-97a1-511348dd37bd")
current_activity.name = "ctapipe-display-muons"
current_activity.startTime = "cta_activity_start_time_utc"
current_activity.endTime = "cta_activity_stop_time_utc"
current_activity.comment = ""
current_activity.activityDescription_id = "ctapipe_display_muons_0.6.1"

res = provClient.addActivity(current_activity)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)

# Association with the agent
wAW = WasAssociatedWith()
wAW.activity = "ed7e27d4-a0a7-43a2-97a1-511348dd37bd"
wAW.agent = "CTAO"

res = provClient.addWasAssociatedWith(wAW)

if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)

current_input_file = DatasetEntity(
    id="hfilename_uuid", classType="dataset", name="logical_name", location="file_url"
)

res = provClient.addDatasetEntity(current_input_file)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)

# Attribution to the agent - wAT.role = ?
wAT = WasAttributedTo(entity="hfilename_uuid", agent="CTAO")

res = provClient.addWasAttributedTo(wAT)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)

# Add the Used relationship
used1 = Used(
    role="cta_input_role",
    activity_id=current_activity.id,
    entity_id=current_input_file.id,
)
res = provClient.addUsed(used1)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)


# Add the wasgeneratedBy relationship - incremental id
wGB1 = WasGeneratedBy(
    role="cta_output_role",
    activity_id=current_activity.id,
    entity_id=current_input_file.id,
)

res = provClient.addWasGeneratedBy(wGB1)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)

# Add the status as an output ValueEntity
current_output_value = ValueEntity(id="cta_activity_uuid_status")
current_output_value.name = "status"
current_output_value.classType = "value"
current_output_value.valueXX = "cta_activity_status"
current_output_value.entityDescription_id = "status"

res = provClient.addValueEntity(current_output_value)
if not res["OK"]:
    DIRAC.gLogger.error(res["Message"])
    DIRAC.exit(-1)



#!/usr/bin/env python

"""
  Expects an input file with a list of lfns with the corresponding transID
  Set transformation files to Processed

  Usage:
    cta-transformation-treat-failed-tasks <ascii file with a list of (transID, lfn)>
"""

__RCSID__ = "$Id$"

# DIRAC imports
import DIRAC
from DIRAC import gLogger
from DIRAC.Core.Base.Script import Script
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

Script.parseCommandLine(ignoreErrors=True)


def read_inputs(file_path):
    content = open(file_path).readlines()
    resList = []
    for line in content:
        transID = line.strip().split(" ")[0]
        lfn = line.strip().split(" ")[1]
        if line != "\n":
            resList.append((transID, lfn))
    return resList


@Script()
def main():
    _, argss = Script.parseCommandLine(ignoreErrors=True)
    if len(argss) != 1:
        Script.showHelp()

    infile = argss[0]
    resList = read_inputs(infile)

    # Initialize the transDict with all transID keys
    transDict = {}
    for transID, lfn in resList:
        transDict.update({transID: []})

    # Update the transDict with values
    for transID, lfn in resList:
        transDict[transID].append(lfn)

    tc = TransformationClient()

    for transID in transDict:
        res = tc.setFileStatusForTransformation(
            transID, newLFNsStatus="Unused", lfns=transDict[transID], force=True
        )
        if not res["OK"]:
            gLogger.error(res["Message"])
            DIRAC.exit(-1)


if __name__ == "__main__":
    main()



"""
 ProvenanceDB test
 December 8th 2021 - P. Maeght
"""

import unittest
from unittest.mock import patch

from CTADIRAC.DataManagementSystem.DB.ProvenanceDB import ProvenanceDB


class TestProvDB(unittest.TestCase):
    """
    test class for ProvDB
    """

    def test_dictToObject(self):
        """
        Test _dictToObject
        """

        def mock_ProvenanceDB__init__(self):
            # No init need for _dictToObject
            pass

        with patch.object(ProvenanceDB, "__init__", mock_ProvenanceDB__init__):
            # New ProvenanceDB
            pdb = ProvenanceDB()

            # Dummy fromDict dict
            item = {
                "field1": "OK",
                "field2": "OK",
            }

            # Table Object
            class Table:
                field1 = "NOK"
                field2 = "NOK"

            # ret = pdb._dictToObject(Table, item)
            pdb._dictToObject(Table, item)

            # Table updated
            assert Table.field2 == b"OK"


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

import json
import os

from DIRAC.Core.Base import Script

Script.parseCommandLine()

# from DIRAC
import DIRAC
from DIRAC.Interfaces.API.Dirac import Dirac
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient

# from CTADIRAC
from CTADIRAC.DataManagementSystem.Client.ProvClient import ProvClient
from CTADIRAC.DataManagementSystem.Client.ProvBase import DatasetEntity
from CTADIRAC.DataManagementSystem.Client.ProvBase import WasAttributedTo
from CTADIRAC.DataManagementSystem.Client.ProvBase import Used
from CTADIRAC.DataManagementSystem.Client.ProvBase import WasGeneratedBy
from CTADIRAC.DataManagementSystem.Client.ProvBase import ValueEntity
from CTADIRAC.DataManagementSystem.Client.ProvBase import Activity
from CTADIRAC.DataManagementSystem.Client.ProvBase import WasAssociatedWith


def get_agent_key(cta_activity):
    res = provClient.getAgentKey("CTAO")
    if not res["OK"]:
        DIRAC.gLogger.error("Agent CTAO not found")
        DIRAC.exit(-1)
    return res["Value"]["internal_key"]


def get_activityDescription_key(cta_activity):
    activityDescription_key = None
    res = provClient.getActvityDescriptionKey(
        cta_activity["activity_name"], cta_activity["system"]["ctapipe_version"]
    )
    if not res["OK"]:
        DIRAC.gLogger.error(res["Message"])
        DIRAC.exit(-1)
    if not res["Value"]["internal_key"]:
        DIRAC.gLogger.error("WARNING: No activity description found")
    else:
        activityDescription_key = res["Value"]["internal_key"]
    return activityDescription_key


def add_activity(cta_activity, activityDescription_key, agent_key=None):
    current_activity = Activity()
    current_activity.id = cta_activity["activity_uuid"]
    current_activity.name = cta_activity["activity_name"]
    current_activity.startTime = cta_activity["start"]["time_utc"]
    current_activity.endTime = cta_activity["stop"]["time_utc"]
    current_activity.comment = ""
    current_activity.activityDescription_key = activityDescription_key

    # Add the activity in yhe database
    res = provClient.addActivity(current_activity)
    if not res["OK"]:
        DIRAC.gLogger.error(res["Message"])
        DIRAC.exit(-1)

    # get the activity internal key
    activity_key = res["Value"]["internal_key"]

    # Association with the agent if specified
    if agent_key:
        wAW = WasAssociatedWith()
        wAW.agent_key = agent_key
        wAW.activity_key = activity_key
        # wAW.role = ?
        res = provClient.addWasAssociatedWith(wAW)
        if not res["OK"]:
            DIRAC.gLogger.error(res["Message"])
            DIRAC.exit(-1)

    return activity_key


def get_usageDescription(cta_activity, role):
    res = provClient.getUsageDescription(cta_activity, role)
    if not res["OK"]:
        DIRAC.gLogger.error(res["Message"])
        DIRAC.exit(-1)
    else:
        return res


def get_generationDescription(cta_activity, role):
    res = provClient.getGenerationDescription(cta_activity, role)
    if not res["OK"]:
        DIRAC.gLogger.error(res["Message"])
        DIRAC.exit(-1)
    else:
        return res


def add_dataset(cta_data, dirac_data, fc, entityDescription_key, agent_key):
    local_file = os.path.basename(cta_data["url"])
    for lfn in dirac_data:
        if local_file == os.path.basename(lfn):
            # Read the DIRAC FileMetadata and Replicas
            res = fc.getFileMetadata(lfn)
            if not res["OK"]:
                DIRAC.gLogger.error(res["Message"])
                DIRAC.exit(-1)
            filename_uuid = res["Value"]["Successful"][lfn]["GUID"]
            creation_date = res["Value"]["Successful"][lfn]["CreationDate"].isoformat()
            res = fc.getReplicas(lfn)
            if not res["OK"]:
                DIRAC.gLogger.error(res["Message"])
                DIRAC.exit(-1)
            location = res["Value"]["Successful"][lfn].keys()

            # Define the DatasetEntity
            current_file = DatasetEntity(
                id=filename_uuid,
                classType="dataset",
                name=lfn,
                location=location,
                generatedAtTime=creation_date,
                entityDescription_key=entityDescription_key,
            )

            # Check if the file already exists
            res = provClient.getDatasetEntity(filename_uuid)
            if not res["OK"]:
                DIRAC.gLogger.error(res["Message"])
                DIRAC.exit(-1)

            if res["Value"]:
                entity_key = res["Value"]
            else:
                res = provClient.addDatasetEntity(current_file)
                if not res["OK"]:
                    DIRAC.gLogger.error(res["Message"])
                    DIRAC.exit(-1)
                entity_key = res["Value"]["internal_key"]

            # Association with the agent if specified
            if agent_key:
                # Attribution to the agent - wAT.role =
                wAT = WasAttributedTo()
                wAT.agent_key = agent_key
                wAT.entity_key = entity_key
                res = provClient.addWasAttributedTo(wAT)
                if not res["OK"]:
                    DIRAC.gLogger.error(res["Message"])
                    DIRAC.exit(-1)

            return entity_key


###############################################################################


def addProvenance(test_VM=None):
    # read provenance dictionary
    f = open("provDict.txt")
    for line in f:
        provStr = line.strip()
    f.close()
    provList = json.loads(provStr.replace("'", '"'))

    if not test_VM:
        # instance of DIRAC
        dirac = Dirac()

        # get the jobID of the DIRAC job
        jobID = os.environ["JOBID"]

        # get inputData of the job
        res = dirac.getJobInputData(jobID)
        if not res["OK"]:
            DIRAC.gLogger.error(res["Message"])
            DIRAC.exit(-1)
        inputData = res["Value"][int(jobID)]

        # get outputData of the job
        # res = dirac.getJobOutputData(jobID)
        # if not res['OK']:
        #  DIRAC.gLogger.error(res['Message'])
        #  DIRAC.exit(-1)
        # outputData = res['Value'][int(jobID)]

    else:
        inputData = [
            "/bidon/proton_20deg_180deg_run22___cta-prod3-demo-2147m-LaPalma-baseline.simtel.gz"
        ]

    outputData = []
    # read output lfns from file
    f = open("output_lfns.txt")
    for line in f:
        outputData.append(line.strip())
    f.close()

    # FileCatalogClient instance used to get guid and location from DFC
    fc = FileCatalogClient()

    # For each activity
    for cta_activity in provList:
        # get Agent key
        agent_key = get_agent_key(cta_activity)

        # get activity description key
        activityDescription_key = get_activityDescription_key(cta_activity)

        # Add the activity in the database and the wasAssociatedWith
        activity_key = add_activity(cta_activity, activityDescription_key, agent_key)

        # For each input file
        for cta_input in cta_activity["input"]:
            # Get entityDescription_key from the UsageDescription
            dict_usageDescription = get_usageDescription(
                activityDescription_key, cta_input["role"]
            )["Value"]
            usageDescription_key = dict_usageDescription["internal_key"]
            entityDescription_key = dict_usageDescription["entityDescription_key"]

            # Add the dataset
            entity_key = add_dataset(
                cta_input, inputData, fc, entityDescription_key, agent_key
            )

            # Add the Used relationship
            #  time = ?
            used = Used(
                role=cta_input["role"],
                activity_key=activity_key,
                entity_key=entity_key,
                usageDescription_key=usageDescription_key,
            )
            res = provClient.addUsed(used)
            if not res["OK"]:
                DIRAC.gLogger.error(res["Message"])
                DIRAC.exit(-1)

        # For each output file
        for cta_output in cta_activity["output"]:
            # Get entityDescription_key from the GenerationDescription
            dict_generationDescription = get_generationDescription(
                activityDescription_key, cta_output["role"]
            )["Value"]
            generationDescription_key = dict_generationDescription["internal_key"]
            entityDescription_key = dict_generationDescription["entityDescription_key"]

            # Add the dataset
            entity_key = add_dataset(
                cta_output, outputData, fc, entityDescription_key, agent_key
            )

            # Add the wasGeneratedBy relationship
            wGB1 = WasGeneratedBy(
                role=cta_output["role"],
                activity_key=activity_key,
                entity_key=entity_key,
                generationDescription_key=generationDescription_key,
            )

            res = provClient.addWasGeneratedBy(wGB1)
            if not res["OK"]:
                DIRAC.gLogger.error(res["Message"])
                DIRAC.exit(-1)

        # Set the status as an output ValueEntity
        if cta_activity["status"]:
            # Get the GenerationDescription
            dict_generationDescription = get_generationDescription(
                activityDescription_key, "status"
            )["Value"]
            generationDescription_key = dict_generationDescription["internal_key"]
            entityDescription_key = dict_generationDescription["entityDescription_key"]

            # Add the ValueEntity
            current_output_value = ValueEntity(
                id=cta_activity["activity_uuid"] + "_status"
            )
            current_output_value.name = "status"
            current_output_value.classType = "value"
            current_output_value.value = cta_activity["status"]
            # current_output_value.generatedAtTime =
            current_output_value.entityDescription_key = entityDescription_key

            res = provClient.addValueEntity(current_output_value)
            if not res["OK"]:
                DIRAC.gLogger.error(res["Message"])
                DIRAC.exit(-1)
            entity_key = res["Value"]["internal_key"]

            # Add the wasGeneratedBy relationship
            wGB1 = WasGeneratedBy(
                role="status",
                activity_key=activity_key,
                entity_key=entity_key,
                generationDescription_key=generationDescription_key,
            )

        res = provClient.addWasGeneratedBy(wGB1)
        if not res["OK"]:
            DIRAC.gLogger.error(res["Message"])
            DIRAC.exit(-1)
        """
        # For each config parameter
        for cta_config_key, cta_config_value in cta_activity['config']['MuonDisplayerTool'].iteritems():
            parameter_id = cta_activity['activity_uuid'] + '_' + cta_config_key
            parameterDescription = provClient.getParameterDescription(activityDescription_key=activityDescription_key, \
                                                                      parameter_name=cta_config_key)
            parameterDescription_key = parameterDescription['Value']['id']
            parameter = Parameter(id=parameter_id, name=cta_config_key, value=cta_config_value, \
                                  parameterDescription_key=parameterDescription_id)
            res = provClient.addParameter(parameter)
            if not res['OK']:
                DIRAC.gLogger.error(res['Message'])
                DIRAC.exit(-1)
            wCB = WasConfiguredBy(artefactType='Parameter', \
                                  activity_key=activity_key, parameter_key=parameter_id)
            res = provClient.addWasConfiguredBy(wCB)
            if not res['OK']:
                DIRAC.gLogger.error(res['Message'])
                DIRAC.exit(-1)
    """
    return DIRAC.S_OK()


###############################################################################
if __name__ == "__main__":
    args = Script.getPositionalArgs()
    try:
        provClient = ProvClient()
        res = addProvenance(args)
        # res = addProvenance()
        if not res["OK"]:
            DIRAC.gLogger.error(res["Message"])
            DIRAC.exit(-1)
        else:
            DIRAC.gLogger.notice("Done")
    except Exception:
        DIRAC.gLogger.exception()
        DIRAC.exit(-1)



import unittest
import CTADIRAC.ProductionSystem.scripts.cta_prod_submit_from_cwl as launcher_cwl


class TestWorkflowStep(unittest.TestCase):
    def test_get_command_line(self):
        wf = launcher_cwl.WorkflowStep()
        input_cwl = "../CWL/setup-software.cwl"
        input_yaml = "../CWL/setup-software.yml"
        wf.get_command_line(input_cwl, input_yaml)
        actual = wf.command_line
        expected = "cta-prod-setup-software -p corsika_simtelarray -v 2022-08-03 -a simulations -g gcc83_matchcpu"
        self.assertEqual(actual, expected)



""""

"""


def extension_metadata():
    return {
        "primary_extension": True,
        "priority": 100,
        "setups": {
            "CTA": "dips://dcta-servers03.pic.es:9135/Configuration/Server,"
            "dips://dcta-agents03.pic.es:9135/Configuration/Server,"
            "dips://ctadirac-01.cscs.cta-observatory.org:9135/Configuration/Server,"
            "dips://ctadirac-02.cscs.cta-observatory.org:9135/Configuration/Server,"
            "dips://ctadirac-03.cscs.cta-observatory.org:9135/Configuration/Server,"
            "dips://ctadirac-04.cscs.cta-observatory.org:9135/Configuration/Server,"
            "dips://ctadirac-05.cscs.cta-observatory.org:9135/Configuration/Server,"
            "dips://cta-dirac.zeuthen.desy.de:9135/Configuration/Server",
            "CTA-cert": "dips://ccdcta-cert.in2p3.fr:9135/Configuration/Server",
        },
        "default_setup": "CTA",
    }



#!/usr/bin/env python
""" Data management script for production
    create DFC MetaData structure put and register files in DFC
    should work for corsika, simtel and EventDisplay output
"""

__RCSID__ = "$Id$"

# generic imports
import os
import glob
import json

# DIRAC imports
import DIRAC
from DIRAC.Core.Base.Script import Script

Script.parseCommandLine()

# Specific DIRAC imports
from CTADIRAC.Core.Utilities.tool_box import run_number_from_filename
from CTADIRAC.Core.Workflow.Modules.ProdDataManager import ProdDataManager


@Script()
def main():
    """simple wrapper to put and register all production files

    Keyword arguments:
    args -- a list of arguments in order []
    """
    args = Script.getPositionalArgs()
    metadata = args[0]
    file_metadata = args[1]
    base_path = args[2]
    output_pattern = args[3]
    package = args[4]
    program_category = args[5]
    catalogs = args[6]
    output_type = args[7]

    # Load catalogs
    catalogs_json = json.loads(catalogs)

    # Create MD structure
    prod_dm = ProdDataManager(catalogs_json)
    result = prod_dm.createMDStructure(
        metadata, base_path, program_category, output_type
    )
    if result["OK"]:
        path = result["Value"]
    else:
        return result

    # Check the content of the output directory
    result = prod_dm._checkemptydir(output_pattern)
    if not result["OK"]:
        return result

    # Dump the list of output LFNs
    file = open("output_lfns.txt", "w")

    # Loop over each file and upload and register
    for localfile in glob.glob(output_pattern):
        file_name = os.path.basename(localfile)
        # Check run number, assign one as file metadata if needed
        fmd_dict = json.loads(file_metadata)
        try:
            run_number = run_number_from_filename(file_name, package)
        except BaseException:
            run_number = -9999
            DIRAC.gLogger.notice("Could not get a correct run number, assigning -9999")
        fmd_dict["runNumber"] = "%08d" % int(run_number)
        # get the output file path
        run_path = prod_dm._getRunPath(fmd_dict)
        lfn = os.path.join(path, output_type, run_path, file_name)
        fmd_json = json.dumps(fmd_dict)
        result = prod_dm.putAndRegister(lfn, localfile, fmd_json, package)
        if not result["OK"]:
            return result
        file.write(lfn)
        file.write("\n")
    file.close()

    DIRAC.exit()


####################################################
if __name__ == "__main__":
    main()
